nUIConfigGUI = LibStub('AceAddon-3.0'):NewAddon('nUIConfigGUI', 'AceEvent-3.0', 'AceConsole-3.0')
local L = LibStub('AceLocale-3.0'):GetLocale('nUIConfigGUI')
local CURRENT_VERSION = GetAddOnMetadata('nUI_Config_GUI', 'Version')

local ADDON_TITLE = GetAddOnMetadata('nUI_Config_GUI', 'Title')
local ADDON_AUTHOR = GetAddOnMetadata('nUI_Config_GUI', 'Author')

local nUI_SlashCmdHandler = SlashCmdList["NUI"]
local _profiling = false

local function getCommandOptions()

	local command_options = {
	    type = "group",
	    args =
		{
			__header1 =
			{
				order = 5,
				type = "header",
				name = "",
			},
			
			__title = {
				type = "description",
				name = ADDON_TITLE,
				order = 6,
			},
			
			__version =
			{
				type = "description",
				name = L["Version"] .. " " .. CURRENT_VERSION .. "\n",
				order = 7,
			},
			
			__author = {
				type = "description",
				name = L["Author"] .. " " .. ADDON_AUTHOR,
				order = 8,
			},
		}
	}

	return command_options

end

local function getConfigHUD()
	local hud = {
		type = "group",
		name = "Настройки HUDа",
		desc = "Настройки HUDа",
		args = {
			__healthrace = {
				type = "toggle",
				name = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_HEALTHRACE].name,
				desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_HEALTHRACE].desc,
				get = function() return nUI_Options.hud_healthrace end,
				set = function(t,b)
					local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
					local option = master.sub_menu[nUI_SLASHCMD_HUD_HEALTHRACE];
					local command = master.command .. " " .. option.command
					if (b) then 
						command = command .. " on"
					else 
						command = command .. " off"
					end
					nUI_SlashCmdHandler(command)
				end,
				width = "full",
				order = 1,
			},
			
			__noncombatnpc = {
				type = "toggle",
				name = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_SHOWNPC].name,
				desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_SHOWNPC].desc,
				get = function() return nUI_Options.show_npc end,
				set = function(t,b) 
					local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
					local option = master.sub_menu[nUI_SLASHCMD_HUD_SHOWNPC];
					local command = master.command .. " " .. option.command
					nUI_SlashCmdHandler(command)
				end,
				width = "full",
				order = 2,
			},
			
			__display = {
				type = "group",
				name = "Вид",
				desc = "Вид",
				args = {
					__scale = {
						type = "range",
						name = "Масштаб",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_SCALE].desc,
						min = 0.25,
						max = 1.75,
						step = 0.05,
						get = function() return nUI_Options.hud_scale end,
						set = function(t, n) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_SCALE];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
					
					__hgap = {
						type = "range",
						name = "Горизонтальный зазор",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_HGAP].desc,
						min = 1,
						max = 1200,
						step = 1,
						get = function() 
							return nUI_Options.hud_hGap 
						end,
						set = function(t, n)
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_HGAP];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)							
						end,
						order = 2
					},
				},
				order = 3,
			},
			
			__alpha = {
				type = "group",
				name = "Прозрачность",
				desc = "Прозрачность HUDа",
				args = {
					__idle = {
						type = "range",
						name = "Простой",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_IDLEALPHA].desc,
						min = 0,
						max = 1,
						step = 0.1,
						get = function() return nUI_Options.hud_alpha.idle end,
						set = function(t, n) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_IDLEALPHA];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
					
					__target = {
						type = "range",
						name = "Цель",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_TARGETALPHA].desc,
						min = 0,
						max = 1,
						step = 0.1,
						get = function() return nUI_Options.hud_alpha.target end,
						set = function(t, n) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_TARGETALPHA];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 2,
					},
					
					__regen = {
						type = "range",
						name = "Регенерация",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_REGENALPHA].desc,
						min = 0,
						max = 1,
						step = 0.1,
						get = function() return nUI_Options.hud_alpha.regen end,
						set = function(t, n)
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_REGENALPHA];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 3,
					},
					
					__combat = {
						type = "range",
						name = "Бой",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_COMBATALPHA].desc,
						min = 0,
						max = 1,
						step = 0.1,
						get = function() return nUI_Options.hud_alpha.combat end,
						set = function(t, n)
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_COMBATALPHA];
							local command = master.command .. " " .. option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 3,
					},					
				
				},
				order = 4,
			},
			
			__cooldown = {
				type = "group",
				name = "Перезарядка",
				desc = "Опции перезарядки",
				args = {
					__bar = {
						type = "toggle",
						name = "Панели",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_COOLDOWN].desc,
						get = function() return nUI_Options.hud_cooldown end,
						set = function(t,b) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_COOLDOWN];
							local command = master.command .. " " .. option.command
							nUI_SlashCmdHandler(command)
						end,
						width = "full",
						order = 1,
					},
					
					__alert = {
						type = "toggle",
						name = "Сигнал тревоги",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_CDALERT].desc,
						get = function() return nUI_Options.hud_cdalert end,
						set = function(t,b) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_CDALERT];
							local command = master.command .. " " .. option.command
							nUI_SlashCmdHandler(command)
						end,
						width = "full",
						order = 2,
					},
					
					__sound = {
						type = "toggle",
						name = "Звук",
						desc = nUI_SlashCommands[nUI_SLASHCMD_HUD].sub_menu[nUI_SLASHCMD_HUD_CDSOUND].desc,
						get = function() return nUI_Options.hud_cdsound end,
						set = function(t,b) 
							local master = nUI_SlashCommands[nUI_SLASHCMD_HUD];
							local option = master.sub_menu[nUI_SLASHCMD_HUD_CDSOUND];
							local command = master.command .. " " .. option.command
							nUI_SlashCmdHandler(command)
						end,
						width = "full",
						order = 3,
					},
				},
				order = 5,
			},
			
		},
	}
	
	return hud
end

local function getConfigTooltips()
	local tooltips = {
		type = "group",
		name = "Опции подсказки",
		desc = nUI_SlashCommands[nUI_SLASHCMD_TOOLTIPS].desc,
		args = {
			__combat = {
				type = "toggle",
				name = "Бой",
				desc = nUI_SlashCommands[nUI_SLASHCMD_COMBATTIPS].desc,
				get = function() return nUI_Options.combat_tooltips end,
				set = function(t,b) 
					local option = nUI_SlashCommands[nUI_SLASHCMD_COMBATTIPS];
					local command = option.command
					nUI_SlashCmdHandler(command)
				end,
				width = "full",
				order = 1,
			},

			__anchor = {
				type = "select",
				name = "Якорь",
				desc = nUI_SlashCommands[nUI_SLASHCMD_TOOLTIPS].desc .. ". ЗАМЕТКА: 'По умолчанию' вызовет перезагрузку вашего UI.",
				values = {
					["owner"] = "Владелец",
					["mouse"] = "Мышь",
					["fixed"] = "Неподвижный",
					["default"] = "По умолчанию"
				},
				style = "dropdown",
				get = function() return nUI_Options.tooltips end,
				set = function(t, s) 
					local option = nUI_SlashCommands[nUI_SLASHCMD_TOOLTIPS]
					local command = option.command .. " " .. s
					nUI_SlashCmdHandler(command)
				end,
				order = 2,
			},
		
		},
	}
	
	return tooltips
end

local function getConfigGeneral()
	local general = {
		type = "group",
		args = {
			__movers = {
				type = "execute",
				name = "Перемещение",
				desc = nUI_SlashCommands[nUI_SLASHCMD_MOVERS].desc,
				func = function() 
					local option = nUI_SlashCommands[nUI_SLASHCMD_MOVERS]				
					local command = option.command
					nUI_SlashCmdHandler(command)
				end,
				-- width = "full",
				order = 1,
			},

			__buttonbag = {
				type = "execute",
				name = "Сумка кнопок",
				desc = nUI_SlashCommands[nUI_SLASHCMD_BUTTONBAG].desc,
				func = function()
					local option = nUI_SlashCommands[nUI_SLASHCMD_BUTTONBAG]
					local command = option.command
					nUI_SlashCmdHandler(command)
				end,
				order = 2,
			},
			
			__animatedportraits = {
				type = "toggle",
				name = "Анимированные портреты",
				desc = nUI_SlashCommands[nUI_SLASHCMD_SHOWANIM].desc,
				get = function() return nUI_Options.show_anim end,
				set = function(t,b) 
					local option = nUI_SlashCommands[nUI_SLASHCMD_SHOWANIM]
					local command = option.command
					nUI_SlashCmdHandler(command)
				end,
				order = 3,
			},
			
			__hplost = {
				type = "toggle",
				name = "Потеря ЗД",
				desc = nUI_SlashCommands[nUI_SLASHCMD_HPLOST].desc,
				get = function() return nUI_Options.hplost end,
				set = function(t,b) 
					local option = nUI_SlashCommands[nUI_SLASHCMD_HPLOST]
					local command = option.command
					nUI_SlashCmdHandler(command)
				end,
				order = 4,
			},

			__clock = {
				type = "group",
				name = "Часы",
				desc = "Опции часов",
				args = {
					__format = {
						type = "toggle",
						name = "24 ч",
						desc = "24 ч",
						get = function() return true end,
						set = function(t,b) end,
						order = 1,
						hidden = true,
					},
					
					__display = {
						type = "select",
						name = "Вид",
						desc = nUI_SlashCommands[nUI_SLASHCMD_CLOCK].desc,
						values = {
							["server"] = "Серверное",
							["local"] = "Местное",
							["both"] = "Оба",
						},
						style = "dropdown",
						get = function() return nUI_Options.clock end,
						set = function(t,s) 
							local option = nUI_SlashCommands[nUI_SLASHCMD_CLOCK]
							local command = option.command .. " " .. s
							nUI_SlashCmdHandler(command)
						end,
						order = 2,
					},
				},
				order = 5,
			},
			
			__bagbar = {
				type = "group",
				name = "Панель сумок",
				desc = "Опции панели сумок",
				args = {
					__active = {
						type = "select",
						name = "Отображение",
						desc = nUI_SlashCommands[nUI_SLASHCMD_BAGBAR].desc,
						values = {
							["on"] = "Вкл",
							["off"] = "Выкл",
							["mouseover"] = "При наводе мыши"
						},
						style = "dropdown",
						get = function() return nUI_Options.bagbar end,
						set = function(t,s)
							local option = nUI_SlashCommands[nUI_SLASHCMD_BAGBAR]
							local command = option.command .. " " .. s
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
					
					__onebag = {
						type = "toggle",
						name = "Одна сумка",
						desc = nUI_SlashCommands[nUI_SLASHCMD_ONEBAG].desc,
						get = function() return nUI_Options.onebag end,
						set = function(t,b)
						local option = nUI_SlashCommands[nUI_SLASHCMD_ONEBAG]
						local command = option.command
						nUI_SlashCmdHandler(command)
						end,
						order = 2,
					},
				},
				order = 6,
			},
			
			__minimap = {
				type = "group",
				name = "Мини-карта",
				desc = "Мини-карта",
				args = {
					__managed = {
						type = "toggle",
						name = "Управление nUI",
						desc = nUI_SlashCommands[nUI_SLASHCMD_MINIMAP].desc,
						get = function() return nUI_Options.minimap end,
						set = function(t,b) 
							local option = nUI_SlashCommands[nUI_SLASHCMD_MINIMAP]
							local command = option.command
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
							
					__shape = {
						type = "select",
						name = "Очертание",
						desc = nUI_SlashCommands[nUI_SLASHCMD_ROUNDMAP].desc,
						values = {
							["round"] = "Круглая",
							["square"] = "Квадратная",
						},
						style = "dropdown",
						get = function()
							local mode = "square"
							if (nUI_Options.round_map) then
								mode = "round"
							end
							return mode
						end,
						set = function(t, s)
							local option = nUI_SlashCommands[nUI_SLASHCMD_ROUNDMAP]
							local command = option.command
							nUI_SlashCmdHandler(command)
						end,
						order = 2,
					},
				},
				order = 7,
				
			},
			
			__display = {
				type = "group",
				name = "Вид",
				desc = "Вид",
				args = {
					__frameupdate = {
						type = "range",
						name = "Частота обновления фрейма",
						desc = nUI_SlashCommands[nUI_SLASHCMD_FRAMERATE].desc,
						min = 1,
						max = 120,
						step = 1,
						get = function() return nUI_Options.frame_rate or 30 end,
						set = function(t,n) 
							local option = nUI_SlashCommands[nUI_SLASHCMD_FRAMERATE]
							local command = option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
					
					__console = {
						type = "select",
						name = "Верхняя консоль",
						desc = nUI_SlashCommands[nUI_SLASHCMD_CONSOLE].desc,
						style = "dropdown",
						values = {
							["mouseover"] = "Мышь",
							["on"] = "Вкл",
							["off"] = "Выкл",
						},
						get = function() return nUI_Options.console end,
						set = function(t, s)
							local option = nUI_SlashCommands[nUI_SLASHCMD_CONSOLE]
							local command = option.command .. " " .. s
							nUI_SlashCmdHandler(command)
						end,
						order = 2,
					}
				
				},
				order = 8,
			},
			
			__unitframes = {
				type = "group",
				name = "Рамки играков",
				desc = "Рамки играков",
				args = {
					__autogroup = {
						type = "toggle",
						name = "Авто группы",
						desc = nUI_SlashCommands[nUI_SLASHCMD_AUTOGROUP].desc,
						get = function() return nUI_Options.auto_group end,
						set = function(t,b)
							local option = nUI_SlashCommands[nUI_SLASHCMD_AUTOGROUP]
							local command = option.command
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
				},
				order = 9,
			},

			__location = {
				type = "group",
				name = "Местонахождение",
				desc = "Опции местонахождения",
				args = {
					__mapcoords = {
						type = "toggle",
						name = "Координаты карты",
						desc = nUI_SlashCommands[nUI_SLASHCMD_MAPCOORDS].desc,
						get = function() return nUI_Options.map_coords end,
						set = function(t,b)
							local option = nUI_SlashCommands[nUI_SLASHCMD_MAPCOORDS]
							local command = option.command
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
				},
				order = 10,
			},
			
			__auras = {
				type = "group",
				name = "Ауры",
				desc = "Опции аур",
				args = {
					__maxauras = {
						type = "range",
						name = "Макс аур",
						desc = nUI_SlashCommands[nUI_SLASHCMD_MAXAURAS].desc,
						min = 1,
						max = 40,
						step = 1,
						get = function() return nUI_Options.max_auras or 40 end,
						set = function(t,n)
							local option = nUI_SlashCommands[nUI_SLASHCMD_MAXAURAS]
							local command = option.command .. " " .. tostring(n)
							nUI_SlashCmdHandler(command)
						end,
						order = 1,
					},
				},
				order = 11,
			},
			
			__profiling = {
				type = "toggle",
				name = "Профили",
				desc = nUI_SlashCommands[nUI_SLASHCMD_LASTITEM+2].desc,
				get = function() return _profiling end,
				set = function(t,b) 
					local option = nUI_SlashCommands[nUI_SLASHCMD_LASTITEM+2]				
					local command = option.command
					nUI_SlashCmdHandler(command)
					_profiling = b
				end,
				order = 99,
				hidden = true,
			},
			
			__debugging = {
				type = "range",
				name = "Отладка",
				desc = nUI_SlashCommands[nUI_SLASHCMD_LASTITEM+1].desc,
				min = 0,
				max = 3,
				step = 1,
				get = function() return nUI_Options.debug end,
				set = function(t,n)
					local option = nUI_SlashCommands[nUI_SLASHCMD_LASTITEM+1]				
					local command = option.command .. " " .. tostring(n)
					nUI_SlashCmdHandler(command)
				end,
				order = 100,	
				-- hidden = true,
			},
			
		},
	}
	
	return general
end
		
function nUIConfigGUI:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
	
	local AceGUI = LibStub("AceGUI-3.0")

	-- Create the options with Ace3
	LibStub("AceConfig-3.0"):RegisterOptionsTable("nUIConfigGUI",getCommandOptions)
	AceConfigReg:RegisterOptionsTable("nUI General",getConfigGeneral)
	AceConfigReg:RegisterOptionsTable("nUI HUD",getConfigHUD)
	AceConfigReg:RegisterOptionsTable("nUI Tooltips",getConfigTooltips)

	self.optionsFrame = AceConfigDialog:AddToBlizOptions("nUIConfigGUI","nUI")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("nUI", "nUI")
	self.optionsFrame[L["General"]] = AceConfigDialog:AddToBlizOptions("nUI General", L["General"], "nUI")		
	self.optionsFrame[L["HUD"]] = AceConfigDialog:AddToBlizOptions("nUI HUD", L["HUD"], "nUI")		
	self.optionsFrame[L["Tooltips"]] = AceConfigDialog:AddToBlizOptions("nUI Tooltips", L["Tooltips"], "nUI")		

	--version update
	if nUIConfigGUIVersion then
		if nUIConfigGUIVersion ~= CURRENT_VERSION then
			self:UpdateSettings(nUIConfigGUIVersion:match('(%w+)%.(%w+)%.(%w+)'))
			self:UpdateVersion()
		end
	--new user
	else
		nUIConfigGUIVersion = CURRENT_VERSION
	end
	
	nUI_SLASHCMD_GUI = #nUI_SlashCommands + 1

	nUI_SlashCommands[nUI_SLASHCMD_GUI] =
	{
		command = "gui",
		options = nil,
		desc    = "Открывает окно настроек nUI GUI",
		message = nil,
	}

	local option = nUI_SlashCommands[nUI_SLASHCMD_GUI]
	nUI_SlashCommands:setHandler( option.command, function() nUIConfigGUI:ChatCommand(); end )

	-- Register slash commands
	self:RegisterChatCommand("nuigui", "ChatCommand")

	nUI_Options.gui = true
	
end

function nUIConfigGUI:OnEnable()
	
end

function nUIConfigGUI:UpdateSettings(major, minor, bugfix)
	--do stuff
end

function nUIConfigGUI:UpdateVersion()
	nUIConfigGUIVersion = CURRENT_VERSION
	self:Print(format(L.Updated, nUIConfigGUIVersion))
end

function nUIConfigGUI:ChatCommand(input)

	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame[L["General"]])

end

